/*
 Chapter 07 - Managing Time Domain
 Timer without delay
 By Francis Perea for Packt Publishing
*/

// Load the notes frequencies definitions
#include "pitches.h"

// Set some global constants
// Total amount of time to wait
#define lapse 3000
// Time to play every note of the chord
#define noteDuration 300
// Time to blink the LED
#define blinkDuration 500

// Global variables we will use
// Speaker pin
int buzzer = 7;
// The number of chrods played
int numberOfSounds = 0;
// LED pin
int LED = 13;
// Initial LED staus
int LEDStatus = LOW;
// The time when the LED blinked last time
unsigned long blinkMark;

// Configuration of the board: two outputs
void setup() {
  pinMode(buzzer, OUTPUT);
  pinMode(LED, OUTPUT);
  // Set the LED initially off
  digitalWrite(LED, LEDStatus);
  // Record the first blink
  blinkMark = millis();
}

// Sketch execution loop
void loop(){
  // If less than 3 chords played
  if (numberOfSounds < 3) {
    // Wait for the desired lapse but blink in the meantime
    myActiveDelay(lapse);
    
    // Play every note and wait blinking
    tone(buzzer, NOTE_G4, noteDuration);
    myActiveDelay(noteDuration);
    
    tone(buzzer, NOTE_A5, noteDuration);
    myActiveDelay(noteDuration);
       
    tone(buzzer, NOTE_C5, noteDuration);
    myActiveDelay(noteDuration);
       
    tone(buzzer, NOTE_D5, noteDuration);
    myActiveDelay(noteDuration);
    
    // Incremet the number of played chords   
    numberOfSounds += 1;
  }
}

void myActiveDelay(int timeLapse){
 // Record the time 
 unsigned long delayMark = millis();
 // wait for the desired timelapse to reach doing something
 while (millis() - delayMark < timeLapse){
   tryToBlinkaLED();
   } 
 }
 
void tryToBlinkaLED(){
   // if the blinking duration has arrived
   if (millis() - blinkMark > blinkDuration){
     // invert the LED status
     LEDStatus = !LEDStatus;
     // turn the LED accordingly
     digitalWrite(LED,LEDStatus);
     // record new blink time
     blinkMark = millis();
   }
}

