/*
 Chapter 07 - Managing Time Domain
 Metronome
 By Francis Perea for Packt Publishing
*/

// Load the notes frequencies definitions
#include "pitches.h"

// Define constants we will use
#define buzzer 9
#define decButton 7
#define incButton 6
#define redLED 5
#define greenLED 4
#define tickDuration 100
#define minBPM 50
#define maxBPM 220
#define pressIncrement 10

// Global variables we will use
// The initial tempo 
int bpm = 60;
int beatDuration;
boolean readButtons;

// Configuration of the board
void setup() {
  pinMode(buzzer, OUTPUT);
  pinMode(decButton, INPUT);
  pinMode(incButton, INPUT);
  pinMode(redLED, OUTPUT);
  pinMode(greenLED, OUTPUT);
}

// Sketch execution loop
void loop(){
  // Allow buttons to be read
  readButtons = true;
  // Compute new beat duration
  beatDuration = ((float)60 / bpm) * 1000;
  // Make beat
  beat();
}

void beat(){
   // Turn LED on
   digitalWrite(redLED, HIGH);
   // Generate half beat sound
   tone(buzzer, NOTE_C3, tickDuration);
   // Wait for sound to finishg while looking at the buttons
   myActiveDelay(beatDuration);
   // Turn LED off
   digitalWrite(redLED, LOW);
   
   //Repeat for the other beat half
   digitalWrite(greenLED, HIGH);
   tone(buzzer, NOTE_B4, tickDuration);
   myActiveDelay(beatDuration);
   digitalWrite(greenLED, LOW);
}

void myActiveDelay(int timeLapse){
 // Record the time 
 unsigned long delayMark = millis();
 // wait for the desired timelapse to reach 
 while (millis() - delayMark < timeLapse){
   //and look at the buttons in the meantime
   if (readButtons == true){
     checkButtons();
   } 
 } 
}
 
void checkButtons(){
  // If haven't reached minimum tempo and decrement button pressed
  if ((digitalRead(decButton) == HIGH) && (bpm > minBPM)){
   // Decrement tempo
   bpm -= pressIncrement;
   // Disable buttons so they are not read again until next beat
   readButtons = false;
  }
  //The same for increment button and maximum tempo
  if ((digitalRead(incButton) == HIGH) && (bpm < maxBPM)){
   bpm += pressIncrement;
   readButtons = false;
  }
}
